package cz.cvut.kbss.reporting.rest;

import cz.cvut.kbss.reporting.dto.OccurrenceReportDto;
import cz.cvut.kbss.reporting.exception.NotFoundException;
import cz.cvut.kbss.reporting.model.OccurrenceReport;
import cz.cvut.kbss.reporting.rest.exception.DataConflictException;
import cz.cvut.kbss.reporting.rest.mapper.DtoMapper;
import cz.cvut.kbss.reporting.rest.util.RestUtils;
import cz.cvut.kbss.reporting.service.repository.OccurrenceReportService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/reports")
public class ReportController extends AbstractController {

    private final OccurrenceReportService reportService;

    private final DtoMapper dtoMapper;

    @Autowired
    public ReportController(OccurrenceReportService reportService, DtoMapper dtoMapper) {
        this.reportService = reportService;
        this.dtoMapper = dtoMapper;
    }

    @RequestMapping(method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public List<OccurrenceReportDto> findAll() {
        final List<OccurrenceReport> reports = reportService.findAll();
        return reports.stream().map(dtoMapper::reportToReportDto).collect(Collectors.toList());
    }

    @RequestMapping(method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> createReport(@RequestBody OccurrenceReport report) {
        reportService.persist(report);
        if (LOG.isDebugEnabled()) {
            LOG.debug("Report {} persisted successfully.", report);
        }
        final HttpHeaders headers = RestUtils.createLocationHeaderFromCurrentUri("/{id}", report.getId());
        return new ResponseEntity<>(headers, HttpStatus.CREATED);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public OccurrenceReport find(@PathVariable("id") Integer reportId) {
        final OccurrenceReport result = reportService.find(reportId);
        if (result == null) {
            throw NotFoundException.create("Report", reportId);
        }
        return result;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public void update(@PathVariable("id") Integer reportId, @RequestBody OccurrenceReport report) {
        if (!reportId.equals(report.getId())) {
            throw new DataConflictException(
                    "Report id " + reportId + " in the URL does not match the report id " + report.getId() +
                            " in the data.");
        }
        if (!reportService.exists(reportId)) {
            throw NotFoundException.create("Report", reportId);
        }
        reportService.update(report);
        if (LOG.isDebugEnabled()) {
            LOG.debug("Report {} updated.", report);
        }
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public void delete(@PathVariable("id") Integer reportId) {
        final OccurrenceReport report = reportService.find(reportId);
        if (report == null) {
            throw NotFoundException.create("Report", reportId);
        }
        reportService.remove(report);
        if (LOG.isDebugEnabled()) {
            LOG.debug("Report {} successfully removed.", report);
        }
    }
}
